#!/bin/sh

PATH="/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:$PATH"
MOUNT_ROOT="$1"
LOG="/dev/null"

OLD_IFS="$IFS"
IFS=':'
for PREFIX in $PATH; do
    [ "x${LVM_TOOL}" = "x" -a -x "$PREFIX/lvm" ] && LVM_TOOL="$PREFIX/lvm"
    [ "x${NTFS3G_TOOL}" = "x" -a -x "$PREFIX/ntfs-3g" ] && NTFS3G_TOOL="$PREFIX/ntfs-3g"
    [ "x${PARTED_TOOL}" = "x" -a -x "$PREFIX/parted" ] && PARTED_TOOL="$PREFIX/parted"
    [ "x${FUSE_EXT2_TOOL}" = "x" -a -x "$PREFIX/fuse-ext2" ] && FUSE_EXT2_TOOL="$PREFIX/fuse-ext2"
    if [ "x${GPART_TOOL}" = "x" -a -x "$PREFIX/gpart" ]; then
        $PREFIX/gpart -V 1>/dev/null 2>&1
        if [ $? -ne 0 ]; then
            GPART_TOOL="$PREFIX/gpart"
        fi
    fi
done
IFS="$OLD_IFS"

do_mount()
{
    echo "$3" | grep -q 'swap' >>"$LOG" 2>&1
    if [ $? -eq 0 ]; then
        return 1
    fi

    #QNAP fix
    if [ -e /etc/config/qpkg.conf ]; then
        if [ "${3}" = "fat16" -o "${3}" = "fat32" ]; then
            FS_TYPE_ARG="-tvfat"
        elif [ "${3}" = "ntfs" ]; then
            FS_TYPE_ARG="-tufsd"
            NTFS3G_TOOL=''
        else
            FS_TYPE_ARG="-t${3}"
        fi
    fi

    #NETGEAR ReadyNAS fix
    if [ -d /proc/readynas ]; then
        if [ "${3}" = "ntfs" ]; then
            FS_TYPE_ARG="-tufsd"
        elif [ "${3}" = "fat32" ]; then
            FS_TYPE_ARG="-tvfat"
        else
            FS_TYPE_ARG="-t${3}"
        fi
    fi

    CREATED=0
    if [ ! -d "$2" ]; then
        mkdir "$2" >>"$LOG" 2>&1
        CREATED=1
    fi

    #re-generate xfs & btrfs UUID
    if [ "${3}" = 'xfs' ]; then
        #xfs_admin -U generate "$1" >>"$LOG" 2>&1
        xfs_db -x -p bhsvc -c 'uuid generate' "$1" 2>&1 | grep ERROR >>"$LOG"
        if [ $? -eq 0 ]; then
            xfs_repair -L "$1" >>"$LOG" 2>&1
            xfs_db -x -p bhsvc -c 'uuid generate' "$1" >>"$LOG" 2>&1
        fi
    elif [ "${3}" = 'btrfs' ]; then
        #repair btrfs ref to original volume
        UUID=`blkid | sed -n '\!^'$1':.*UUID="\([^"]\+\)".*!s!!\1!p'`
        ORIGIN=`blkid | grep -v $1 | sed -n '\!^\([^:]\+\):.*UUID="'$UUID'".*!s!!\1!p' | head -n 1`
        if [ "x$ORIGIN" != "x" ]; then
            btrfs device scan "$ORIGIN" >>"$LOG" 2>&1 && \
            btrfstune -f -u "$1" >>"$LOG" 2>&1 && \
            (echo Y | btrfsck --repair "$1" >>"$LOG" 2>&1 )
            if [ $? -ne 0 ]; then
                return 100
            fi
        fi
    fi

    mount 2>/dev/null | grep -qE "^$1\$"
    if [ $? -eq 0 ]; then
        return 0
    fi

    FSOPTIONS='ro,noauto,nodev,nosuid,noexec,noatime,nodiratime'
    if [ "${3}" = 'freebsd-ufs' ]; then
        if [ "x${4}" = 'xufs' ]; then
            FSOPTIONS="ro"
            FS_TYPE_ARG='-t ufs'
        else
            FSOPTIONS="$FSOPTIONS,ufstype=ufs2"
        fi
    elif [ "${3}" = "fat32" -a "x$4" = "xmsdosfs" ]; then
        FSOPTIONS='ro,noauto,nosuid,noexec,noatime'
        FS_TYPE_ARG='-t msdosfs'
    elif [ "${3}" = "fat16" -o "${3}" = "fat32" ]; then
        FSOPTIONS="$FSOPTIONS,utf8"
    fi

    if [ "${3}" = 'ntfs' -a "x${NTFS3G_TOOL}" != "x" ]; then
        "$NTFS3G_TOOL" -o "ro,permissions" "$1" "$2" >>"$LOG" 2>&1
        RESULT=$?
    elif [ "x${4}" = "xext2fs" -a "x${FUSE_EXT2_TOOL}" != "x" ]; then
        "$FUSE_EXT2_TOOL" -o 'ro' "$1" "$2" >>"$LOG" 2>&1
        RESULT=$?
    else
        eval mount ${FS_TYPE_ARG} -o "${FSOPTIONS}" "${1}" "${2}" >>"$LOG" 2>&1
        RESULT=$?

        if [ $RESULT -ne 0 -a \( "${3}" = 'ext4' -o "${3}" = 'xfs' \) ]; then
             eval mount ${FS_TYPE_ARG} -o "$FSOPTIONS,norecovery" "$1" "$2" >>"$LOG" 2>&1
             RESULT=$?
        fi
    fi

    if [ $RESULT -ne 0 -a $CREATED -eq 1 ]; then
        rm -rf "$2" >>"$LOG" 2>&1
    fi

    return $RESULT
}

TEST_PARTITIONS=""
TEST_PARTITIONS_FILTER=""

shift
for DEVICE in "$@"; do
    if [ ! -e "$DEVICE" ]; then
        SKIP=1
    elif ( echo "$DEVICE" | grep -qE '(-real)|(-cow)|(_rmeta_[0-9]+)|(_rimage_[0-9]+)$' ); then
        SKIP=1
    elif [ "x${LVM_TOOL}" != "x" ]; then
        "$LVM_TOOL" lvs --noheadings --separator ':' -o lv_attr "$DEVICE" 2>/dev/null | grep -qE '^[[:space:]]*[sS].*'
        if [ $? -eq 0 ]; then
            SKIP=1
        fi
    fi
    if [ ${SKIP:-0} -eq 1 ]; then
        echo "Skipping $DEVICE" >>"$LOG" 2>&1
        continue
    fi

    if [ "x${PARTED_TOOL}" != "x" ]; then
        TYPE=`"$PARTED_TOOL" -s -m "$DEVICE" unit B print 2>>"$LOG" | sed -n '/.*\(msdos\|gpt\|loop\).*/s//\1/p'`
        PARTITIONS=`"$PARTED_TOOL" -s -m "$DEVICE" unit B print 2>>"$LOG" | sed -n '/^\([[:digit:]]\+.*\);$/s//\1/p' | grep -v msftres | cut -d : -f 1,4,5 | sed -e  's/B:/:/'`
        if [ "x${PARTITIONS}" = "x" ]; then
            PARTITIONS=`fdisk --bytes -l "$DEVICE" 2>>"$LOG" |  sed -n '\!^\(/dev/[a-z]\+\([0-9]\+\)\)[[:space:]*]\+[0-9]\+[[:space:]]\+[0-9]\+[[:space:]]\+[0-9]\+[[:space:]]\+\([0-9]\+\)[[:space:]+]\+\(82\|83\|b\|7\).*$!s!!\2:\3=\4!p' | sed -e 's/=82/:swap/' -e 's/=83/:ext4/' -e 's/=b/:fat32/' -e 's/=7/:ntfs/'`
            if [ "x${PARTITIONS}" = "x" ]; then
                PARTITIONS=`fdisk -l "$DEVICE" 2>>"$LOG" | sed -n '\!^\(/dev/.*\([0-9]\+\)\)[[:space:]*]\+[0-9]\+[[:space:]]\+[0-9]\+[[:space:]]\+\([0-9]\+\)[[:space:]+]\+\(82\|83\|b\|7\).*$!s!!\2:\3=\4!p' | sed -e 's/=82/:swap/' -e 's/=83/:ext4/' -e 's/=b/:fat32/' -e 's/=7/:ntfs/'`
                if [ "x${PARTITIONS}" = "x" ]; then
                    DEVICE_NAME=`ls -la $DEVICE | sed -n '\!.*/\([^/]\+\)$!s!!\1!p'`
                    SIZE=$((`cat "/sys/block/$DEVICE_NAME/size" 2>>"$LOG"` * 512))
                    FSTYPE=`file -bsL "$DEVICE" 2>>"$LOG" | \
                            sed -e 's/Linux rev [^ ]* //'\
                                -e 's/Linux\/[^ ]* //' -e 's/SGI //' \
                                -e '/^.*, FAT ([1-3][0-9] bit).*/s//fat32/' \
                                -e '/^.*OEM-ID[ =]"NTFS.*/s//ntfs/' \
                                -e '/^Unix Fast File system.*/s//freebsd-ufs/' \
                                -e '/\(ext[2-4]\).*/s//\1/' | \
                            tr 'A-Z' 'a-z' | \
                            grep -vi lvm | \
                            cut -d ' ' -f 1`
                    if [ "x${FSTYPE}" != "x" ]; then
                        PARTITIONS="$DEVICE:${SIZE:-0}:$FSTYPE"
                    else
                        TEST_PARTITIONS="$TEST_PARTITIONS $DEVICE"
                        if [ "x$TEST_PARTITIONS_FILTER" = "x" ]; then
                            TEST_PARTITIONS_FILTER="(${DEVICE}:)"
                        else
                            TEST_PARTITIONS_FILTER="${TEST_PARTITIONS_FILTER}|(${DEVICE}:)"
                        fi
                        continue
                    fi
                fi
            fi
            TYPE="msdos"
        fi
    elif [ "x${GPART_TOOL}" != "x" ]; then
        PARTS=`"$GPART_TOOL" show -p $DEVICE | sed -n -e '1d' -e '/- free -/d' -e '\@^[[:space:]]*[0-9]*[[:space:]]*\([0-9]*\)[[:space:]]*\([[:alnum:]]\{1,\}\).*$@s@@/dev/\2 \1@p'`
        IFS='
'
        for PARTITION in $PARTS; do
            PARTITION_DEVICE=`echo ${PARTITION} | cut -d ' ' -f 1`
            SIZE=$((`echo $PARTITION | cut -d ' ' -f 2` * 512))
            FSTYPE=`file -bsL "$PARTITION_DEVICE" | \
                        sed -e 's/Linux rev [^ ]* //'\
                            -e 's/Linux\/[^ ]* //' -e 's/SGI //' \
                            -e '/^.*, FAT (32 bit).*/s//fat32:msdosfs/' \
                            -e '/^.*OEM-ID[ =]"NTFS.*/s//ntfs/' \
                            -e '/^Unix Fast File system.*/s//freebsd-ufs:ufs/' \
                            -e '/\(ext[2-4]\).*/s//\1:ext2fs/' | \
                        tr 'A-Z' 'a-z' | \
                        cut -d ' ' -f 1`

            if [ "x$PARTITIONS" = "x" ]; then
                PARTITIONS="$PARTITION_DEVICE:$SIZE:$FSTYPE"
            else
                PARTITIONS="$PARTITIONS\n$PARTITION_DEVICE:$SIZE:$FSTYPE"
            fi
        done
        IFS="$OLD_IFS"
        PARTITIONS=`printf "$PARTITIONS"`
    fi

    IFS='
'
    for PARTITION in ${PARTITIONS}; do
        INDEX=`echo $PARTITION | cut -d : -f 1`
        SIZE=`echo $PARTITION | cut -d : -f 2`
        FSTYPE=`echo $PARTITION | cut -d : -f 3`
        HINT=`echo $PARTITION | cut -d : -f 4`

        echo "$INDEX" | grep -qE '^/dev/'
        if [ $? -eq 0 -a -e "$INDEX" ]; then
            PARTITION_DEVICE="$INDEX"
        elif [ "x${TYPE}" = "xloop" ]; then
            if [ ${INDEX:=1} -ne 1 ]; then
                break
            fi
            PARTITION_DEVICE="$DEVICE"
        else
            PARTITION_DEVICE="${DEVICE}${INDEX}"

            if [ ! -e "$PARTITION_DEVICE" ]; then
                PARTITION_DEVICE="${DEVICE}p${INDEX}"
            fi

            if [ ! -e "$PARTITION_DEVICE" ]; then
                PARTITION_DEVICE="${DEVICE}s${INDEX}"
            fi
        fi

        if [ "x${FSTYPE}" = "x" ]; then
            TEST_PARTITIONS="$TEST_PARTITIONS $PARTITION_DEVICE"
            if [ "x$TEST_PARTITIONS_FILTER" = "x" ]; then
                TEST_PARTITIONS_FILTER="(${PARTITION_DEVICE}:)"
            else
                TEST_PARTITIONS_FILTER="${TEST_PARTITIONS_FILTER}|(${PARTITION_DEVICE}:)"
            fi
            continue
        fi

        MOUNT=`basename "$PARTITION_DEVICE"`
        do_mount "$PARTITION_DEVICE" "$MOUNT_ROOT/$MOUNT" "$FSTYPE" "${HINT}"
        if [ $? -ne 0 ]; then
            echo "$PARTITION_DEVICE $SIZE $FSTYPE"
        else
            echo "$PARTITION_DEVICE $SIZE $FSTYPE $MOUNT_ROOT/$MOUNT"
        fi
    done
    IFS="$OLD_IFS"
done

if [ "x${TEST_PARTITIONS}" != "x" -a "x${LVM_TOOL}" != "x" ]; then
    VGS=`eval "$LVM_TOOL" pvs --noheadings --separator ':' -o pv_name,vg_name,lv_attr $TEST_PARTITIONS 2>/dev/null | grep -E "$TEST_PARTITIONS_FILTER" | sed -n '/[^:]\+:\([^:]\+\):....-.*$/s//\1/p' | sort -u`
    IFS='
'
    for VG in $VGS; do
        "$LVM_TOOL" vgchange -a ey "$VG" >>"$LOG" 2>&1
    done
fi

exit 0
