#!/bin/sh

PATH="/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:$PATH"
MOUNT_ROOT="$1"
LOG="/dev/null"

OLD_IFS="$IFS"
IFS=':'
for PREFIX in $PATH; do
    [ "x${LVM_TOOL}" = "x" -a -x "$PREFIX/lvm" ] && LVM_TOOL="$PREFIX/lvm"
    [ "x${DMSETUP_TOOL}" = "x" -a -x "$PREFIX/dmsetup" ] && DMSETUP_TOOL="$PREFIX/dmsetup"
done
IFS="$OLD_IFS"

shift
for DEVICE in "$@"; do
    MOUNT=`basename "$DEVICE"`
    if [ -d "$MOUNT_ROOT/$MOUNT" ]; then
        umount "$MOUNT_ROOT/$MOUNT" >>"$LOG" 2>&1
        rmdir "$MOUNT_ROOT/$MOUNT" >>"$LOG" 2>&1
    fi

    if [ "x${LVM_TOOL}" != "x" ]; then
        IFS='
'
        VGS=`eval "$LVM_TOOL" pvs --noheadings --separator ':' -o pv_name,vg_name,lv_attr 2>/dev/null | grep -E "${DEVICE}" | sed -n '/[^:]\+:\([^:]\+\):....a.*$/s//\1/p' | sort -u`
        for VG in $VGS; do
            "$LVM_TOOL" vgchange -a n "$VG" >>"$LOG" 2>&1
        done
        IFS="$OLD_IFS"
    fi

    echo "$DEVICE" | grep -q '/dev/mapper/' >>"$LOG" 2>&1
    if [ $? -eq 0 -a "x${DMSETUP_TOOL}" != "x" ]; then
        "$DMSETUP_TOOL" remove "${DEVICE}" >>"$LOG" 2>&1
    fi
done

#cleanup devmapper internal leftovers
for DEVICE in "$@"; do
    echo "$DEVICE" | grep -q '/dev/mapper/' >>"$LOG" 2>&1
    if [ $? -eq 0 -a "x${DMSETUP_TOOL}" != "x" ]; then
        for SUFFIX in _mlog _mimage_0 _mimage_1 _mimage_2 _mimage_3 _mimage_4 _mimage_5 _mimage_6 _mimage_7 _mimage_8 _mimage_9 -real -cow; do
            [ -e "${DEVICE}${SUFFIX}" ] && "$DMSETUP_TOOL" remove "${DEVICE}${SUFFIX}" >>"$LOG" 2>&1
        done
    fi
done

exit 0
