#!/bin/sh

PATH="/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:$PATH"
PORTAL="$1"
TARGET="$2"
LOG="/dev/null"

resolve_iscsi_device()
{
    if [ "x${ISCSI_CTL}" != "x" ]; then
        DEVICE=`$ISCSI_CTL -L | grep "$TARGET" | sed -n '/.* Connected: \(.*\)$/s//\1/p'`
    else
        DEVICE=`iscsiadm -m session --print=3 2>&1 | grep -E '(Target:)|(Attached scsi disk)' | grep -A 1 "Target: $TARGET" | sed -n '/[[:space:]]*Attached scsi disk \([[:alnum:]]\+\).*/s//\1/p'`
        if [ "x${DEVICE}" = "x" ]; then
            SCSI_ID=`iscsiadm -m session --print=3 2>&1 | grep -E '(Target:)|(Lun:)' | grep -A 1 "Target: $TARGET" | sed -n '/[[:space:]]*scsi\([0-9]\+\) Channel \(0\{0,1\}\)\([0-9]\+\) Id \([0-9]\+\) Lun: \([0-9]\+\).*/s//\1:\3:\4:\5/p'`
            if [ "x${SCSI_ID}" != "x" ]; then
                DEVICE=`ls -la /sys/block/*/device | grep -E "/${SCSI_ID}/?$" | sed -n '\@.*/sys/block/\([^/]*\)/.*@s@@\1@p'`
            fi
        fi
    fi
}

#QNAP fix
if [ -e /etc/config/qpkg.conf ]; then
    PORTAL=`echo $PORTAL | sed '/\([^:]*\):\(.*\)/s//[\1]:\2/'`
elif [ -x '/usr/bin/iscsictl' ]; then
    #BSD tools
    ISCSI_CTL='/usr/bin/iscsictl'
fi

resolve_iscsi_device
if [ "x${DEVICE}" = "x" ]; then
    if [ "x${ISCSI_CTL}" != "x" ]; then
        $ISCSI_CTL -A -p "$PORTAL" -t "$TARGET" -e on >>"$LOG" 2>&1
    else
        iscsiadm -m discovery -p "$PORTAL" -t sendtargets >>"$LOG" 2>&1
        if  [ $? -ne 0 ]; then
            exit 1
        fi

        iscsiadm -m node -p "$PORTAL" -T "$TARGET" -l >>"$LOG" 2>&1
        if  [ $? -ne 0 ]; then
            exit 2
        fi
    fi

    ATTEMPT=0
    while [ "x$DEVICE" = "x" -a $ATTEMPT -lt 10 ]; do
        resolve_iscsi_device
        if [ "x$DEVICE" = "x" ]; then
            ATTEMPT=`expr $ATTEMPT + 1`
            sleep 1 >>"$LOG" 2>&1
        fi
    done
fi

if [ "x${DEVICE}" = "x" ]; then
    scripts/iscsi_unmount.sh "$PORTAL" "$TARGET"
    exit 3
fi

echo "/dev/$DEVICE"
exit 0
