#!/bin/sh

PATH="/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:$PATH"
MOUNT="$1"
URI="$2"
DOMAIN="$3"
USER="$4"
PASSWORD="$5"
OPTIONS="${6}"
CREATE="${7:-no}"
BHUID="${8:-bhsvc}"
BHGID="${9:-bhsvc}"
LOG="/dev/null"

mount 2>/dev/null | grep -q "on $MOUNT " >>"$LOG" 2>&1
if [ $? -eq 0 -a -d "$MOUNT" ]; then
    echo "$MOUNT"
    exit 0
fi

if [ ! -d "$MOUNT" ]; then
    mkdir "$MOUNT" >>"$LOG" 2>&1
fi

PROTO=`echo $URI | cut -d ':' -f 1`
SHARE=`echo $URI | cut -d ':' -f 2-`
case "$PROTO" in
    "cifs")
        USER_OPTIONS=",uid=$BHUID,gid=$BHGID"

        #Asustor fix
        if [ -e /usr/sbin/apkg ]; then
            MOUNT_CIFS='/usr/builtin/sbin/mount.cifs'
            USER_OPTIONS=""
            ADM_VERSION=`sed -n '/^Version = \([0-9]\)\..*$/s//\1/p' /etc/nas.conf 2>/dev/null`
            if [ "x$ADM_VERSION" = "x" -o $ADM_VERSION -lt 4 ]; then
                #ADM 3.0
                FORCE_VERSION_NONE=1
            fi
        elif [ -e /usr/sbin/mount_smbfs ]; then
            #BSD tools
            MOUNT_SMBFS='/usr/sbin/mount_smbfs'
        else
            MOUNT_CIFS='/sbin/mount.cifs'
        fi

        if [ "x${MOUNT_SMBFS}" != "x" ]; then
            HOST=`echo $URI | cut -d '/' -f 3`
            expect  -c "spawn $MOUNT_SMBFS -u $BHUID -g $BHGID  -U $USER -I $HOST $SHARE $MOUNT" \
                    -c 'expect "Password:"' \
                    -c "send \"$PASSWORD\r\"" \
                    -c 'expect eof' \
                    -c 'lassign [wait] a1 a2 a3 a4' \
                    -c 'exit $a4' >>"$LOG" 2>&1
            RESULT=$?
        elif [ "x$OPTIONS" != "x" ]; then
            PASSWD="$PASSWORD" "$MOUNT_CIFS" -o "domain=$DOMAIN,username=$USER,$OPTIONS$USER_OPTIONS" "$SHARE" "$MOUNT" >>"$LOG" 2>&1
            RESULT=$?
        else
            ACCESS="file_mode=0666,dir_mode=0777"
            if [ "x$CREATE" = "xro" ]; then
                ACCESS="ro,file_mode=0444,dir_mode=0555"
            fi
            OPTIONS="domain=$DOMAIN,username=$USER,iocharset=utf8,nounix,nocase,noserverino,iocharset=utf8,$ACCESS$USER_OPTIONS"

            if [ -x "$MOUNT_CIFS" ]; then
                OPTIONS="$OPTIONS,cache=loose,nobrl"
                VERSIONS='none'
                if [ ${FORCE_VERSION_NONE:-0} -ne 1 ]; then
                    KERNEL_MAJOR=`uname -r 2>/dev/null | cut -d '.' -f 1`
                    KERNEL_MINOR=`uname -r 2>/dev/null | cut -d '.' -f 2`
                    if [ \( $KERNEL_MAJOR -eq 3 -a $KERNEL_MINOR -ge 7 \) -o $KERNEL_MAJOR -ge 4 ]; then
                        VERSIONS="2.1 2.0 1.0"
                        if [ \( $KERNEL_MAJOR -eq 4 -a $KERNEL_MINOR -ge 4 \) -o $KERNEL_MAJOR -ge 5 ]; then
                            VERSIONS="3.02 3.0 $VERSIONS"
                        fi
                    fi
                fi

                for VERSION in $VERSIONS; do
                    if [ "x$VERSION" = "xnone" ]; then
                        VERSION=''
                        PASSWD="$PASSWORD" "$MOUNT_CIFS" -o "${OPTIONS}" "$SHARE" "$MOUNT" >>"$LOG" 2>&1
                        RESULT=$?
                    else
                        VERSION="vers=${VERSION},"
                        RESULT=1
                    fi

                    if [ $RESULT -ne 0 ]; then
                        for SECURITY in ntlmssp ntlmv2 ntlmsspi ntlmv2i; do
                            PASSWD="$PASSWORD" "$MOUNT_CIFS" -o "${VERSION}sec=${SECURITY},${OPTIONS}" "$SHARE" "$MOUNT" >>"$LOG" 2>&1
                            RESULT=$?
                            if [ $RESULT -eq 0 ]; then break; fi
                        done
                        if [ $RESULT -eq 0 ]; then break; fi
                    fi
                done
            else
                mount -t cifs -o "password=$PASSWORD,$OPTIONS" "$SHARE" "$MOUNT" >>"$LOG" 2>&1
                RESULT=$?
            fi
        fi
    ;;
    "nfs")
        if [ "x$OPTIONS" != "x" ]; then
            if [ "x$OPTIONS" = "xddboost" ]; then
                CUSTOM_TOOL='/opt/emc/boostfs/bin/boostfs'

                mount 2>/dev/null | grep -q "on ${MOUNT}.ddboost " >>"$LOG" 2>&1
                if [ $? -eq 0 -a -d "${MOUNT}.ddboost" ]; then
                    echo "${MOUNT}.ddboost"
                    exit 0
                fi

                mv "$MOUNT" "${MOUNT}.ddboost" >>"$LOG" 2>&1
                MOUNT="${MOUNT}.ddboost"

                ADDRESS="${SHARE%:/*}"
                STORAGE_UNIT="${SHARE##*:/}"

                ${CUSTOM_TOOL} mount -o allow-others=true -d "${ADDRESS}" -s "${STORAGE_UNIT}" "$MOUNT" >>"$LOG" 2>&1
            elif ( echo "$OPTIONS" | grep -qE '^hsfs' ); then
                CUSTOM_TOOL='mount.hsfs'
                OPTIONS=`echo "$OPTIONS" | sed '/^hsfs,\{0,1\}\(.*\)$/s//\1/'`

                if [ "x$OPTIONS" = "x" ]; then
                    rmdir "$MOUNT" >>"$LOG" 2>&1
                    MOUNT="${MOUNT}.link"

                    if [ ! -e "$MOUNT" ]; then
                        ln -s "${SHARE##*:}" "$MOUNT" >>"$LOG" 2>&1
                    else
                        test -L "$MOUNT" >>"$LOG" 2>&1
                    fi
                else
                    mount 2>/dev/null | grep -q "on ${MOUNT}.hsfs " >>"$LOG" 2>&1
                    if [ $? -eq 0 -a -d "${MOUNT}.hsfs" ]; then
                        echo "${MOUNT}.hsfs"
                        exit 0
                    fi

                    mv "$MOUNT" "${MOUNT}.hsfs" >>"$LOG" 2>&1
                    MOUNT="${MOUNT}.hsfs"

                    ${CUSTOM_TOOL} -o "$OPTIONS" "$SHARE" "$MOUNT" >>"$LOG" 2>&1
                fi
            else
                mount -t nfs -o "$OPTIONS" "$SHARE" "$MOUNT" >>"$LOG" 2>&1
            fi
            RESULT=$?
        else
            NFS_CLIENT="hard"
            if [ "x$CREATE" = "xro" ]; then
                NFS_CLIENT="soft,retrans=5,timeo=600,ro"
            fi

            mount -t nfs -o "${NFS_CLIENT},proto=tcp,intr,nosuid,noexec" "$SHARE" "$MOUNT" >>"$LOG" 2>&1
            RESULT=$?

            if [ $RESULT -ne 0 ]; then
                mount -t nfs -o "${NFS_CLIENT},nfsvers=3,proto=tcp,intr,nosuid,noexec" "$SHARE" "$MOUNT" >>"$LOG" 2>&1
                RESULT=$?

                if [ $RESULT -ne 0 ]; then
                    mount -t nfs -o "${NFS_CLIENT},nfsvers=2,proto=tcp,intr,nosuid,noexec" "$SHARE" "$MOUNT" >>"$LOG" 2>&1
                    RESULT=$?
                fi
            fi
        fi

        if [ $RESULT -eq 0 -a "x$CREATE" = "xyes" ]; then
            if [ -e "$MOUNT/NakivoBackup" ]; then
                ${CUSTOM_TOOL} umount "$MOUNT" >>"$LOG" 2>&1
                RESULT=101
            else
                mkdir "$MOUNT/NakivoBackup" >>"$LOG" 2>&1
                chown $BHUID:$BHGID "$MOUNT/NakivoBackup" >>"$LOG" 2>&1
                chmod 770 "$MOUNT/NakivoBackup" >>"$LOG" 2>&1
            fi
        fi
    ;;
    *)
        RESULT=100
    ;;
esac

if [ $RESULT -ne 0 ]; then
    rmdir "$MOUNT" >>"$LOG" 2>&1
    echo $RESULT
    exit 255
fi

echo "$MOUNT"
exit 0
